
# // Space class
class Space(object):
    def __init__(self, type: str = "v", size: int = 1) -> None:
        self.type = type
        self.size = size
        
    def __str__(self) -> str:
        return f"\\{self.type}space{{{self.size}cm}}"

    


from dataclasses import dataclass
import logging

logging.basicConfig(
    format = '%(asctime)s,%(msecs)03d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s',
    datefmt = '%Y-%m-%d:%H:%M:%S',
    level = logging.DEBUG
)

@dataclass
class Errors:
    def warning(msg: str) -> str:
        logging.warning(msg)
        return msg

    def critical(msg: str) -> str:
        logging.warning(msg)
        return msg



# // Flex class
class Flex(object):
    def __init__(self, width: int = 0.33, items: list[any] = []) -> None:
        self.width = width
        self.items = items
        
    def __str__(self) -> str:
        return "".join((
            f"\\begin{{minipage}}{{{self.width}\\textwidth}}" + str(i) +
            f"\\end{{minipage}}\\leavevmode"
        ) for i in self.items)



from dataclasses import dataclass

# // Packages Dataclass
@dataclass
class Packages:
    multirow: str = "multirow"
    cancel: str = "cancel"
    changepage: str = "changepage"
    amssymb: str = "amssymb"
    amsmath: str = "amsmath"
    gensymb: str = "gensymb"
    tkzeuclide: str = "tkz-euclide"
    graphicx: str = "graphicx"



from .errors import Errors

# // Table class
class Table(object):
    def __init__(self, columns: int = 0, headers: list[str] = [], data: list[any] = []) -> None:
        self.headers = headers
        self.columns = columns
        self.data = data
        
    def __str__(self) -> str:
        if len(self.data) < self.columns:
            return Errors.critical(f"Not enough data in table with '{self.columns}' columns!")
        
        if self.columns > 0:
            self.data = self._split_data(self.data)
            return (
                "\\begin{tabular}{" + "".join('|c' for _ in range(self.columns)) + "|}\hline" + 
                self._headers() + 
                self._body() + 
                "\\end{tabular}\\leavevmode\\\\"
            )
        Errors.warning("Table is empty. (0 columns)")
        return ""
        
    def _split_data(self, data: list[any]) -> list[any]:
        return [data[x:x + self.columns] for x in range(0, len(data), self.columns)]
    
    def _headers(self) -> str:
        for i in range(len(self.headers)):
            if i != len(self.headers) - 1:
                self.headers[i] = f"{{{self.headers[i]}}}&"
            else:
                self.headers[i] = f"{{{self.headers[i]}}}\\\\\\hline"
        return "".join(h for h in self.headers)
    
    def _body(self) -> str:
        for i in range(len(self.data)):
            for j in range(len(self.data[i])):
                if j != len(self.data[i]) - 1:
                    self.data[i][j] = f"{{{self.data[i][j]}}}&"
                else:
                    self.data[i][j] = f"{{{self.data[i][j]}}}\\\\\\hline"
        return "".join("".join(a for a in b) for b in self.data)




# // Align class
class Align(object):
    # // flalign, fralign, align
    def __init__(self, type: str = "align", items: list[any] = []):
        self.type = type
        self.items = items

    def __str__(self):
        # \begin{flalign*}
        pass

# // Adjust class
class Adjust(object):
    def __init__(self, width: int = 0, margin: int = 0, items: list[any] = []) -> None:
        self.width = width
        self.margin = margin
        self.items = items
    
    def __str__(self) -> str:
        if len(self.items) > 0:
            return (
                f"\\begin{{adjustwidth}}{{{self.width}cm}}{{{self.margin}pt}}" +
                "".join(str(i) for i in self.items) +
                f"\\end{{adjustwidth}}\\leavevmode\\\\"
            )
        return ""



from .build import *
from .image import *
from .space import *
from .header import *
from .list import *
from .adjust import *
from .packages import *
from .table import *
from .text import *
from .flex import *



# // Header class
class Header(object):
    def __init__(self, content: str = "", enumerate: bool = False) -> None:
        self.content = content
        self.enumerate = enumerate
        
    def __str__(self) -> str:
        return f"\section{self.enumerate}{{{self.content}}}"




# // Image class
class Image(object):
    def __init__(self, path: str, scale: int = 1) -> None:
        self.path = path
        self.scale = scale
    
    def __str__(self) -> str:
        return f"\includegraphics[scale={self.scale}]{{{self.path}}}"
    


import re

# // Equations class
class Eq(object):
    def __init__(self):
        pass

    def __str__(self) -> str:
        pass


# // Text class
class Text(object):
    def __init__(self, content: str = "", bold: bool = False, italic: bool = False) -> None:
        self.content = re.sub("\s\s+", " ", content).strip()
        self.bold = bold
        self.italic = italic
    
    def clean_content(self, content: str) -> str:
        active = False
        content = list(content)
        for i in range(len(content)):
            if content[i] == "$":
                active = not active

            if active:
                if content[i] == " ":
                    content[i] = "$ $"
        return r"".join(c for c in content) + " "
        
    def __str__(self) -> str:
        self.content = self.clean_content(self.content)
        if self.bold:
            return f"\\textbf{{{self.content}}}"
        elif self.italic:
            return f"\\textit{{{self.content}}}"
        elif self.italic and self.bold:
            return f"\\textbf{{\\textit{{{self.content}}}}}"
        return f"{{{self.content}}}"
    




# // List class
class List(object):
    def __init__(self, type: str = "itemize", items: list[any] = []) -> None:
        self.type = type
        self.items = items
        
    def __str__(self) -> str:
        return (
            f"\\begin{{{self.type}}}" +
            "".join(f"\\item {{{i}}}" for i in self.items) +
            f"\\end{{{self.type}}}\\leavevmode"
        )



from .adjust import Align, Adjust
from .header import Header
from .table import Table
from .image import Image
from .space import Space
from .list import List
from .text import Text
from .flex import Flex
import os, shutil

# // Build class
class Build:
    # // Initialize the class and build folder
    def __init__(self, build_file: str = "main.tex", file_dir: str = ""):
        self.build_file = build_file
        self.file_dir = ""
        
        if len(file_dir) > 0:
            self.file_dir = os.path.dirname(os.path.abspath(file_dir))
            
        if not os.path.exists(f"{self.file_dir}/build"):
            os.mkdir(f"{self.file_dir}/build")

    # // Initialize the PDF Document
    def new(self, 
            doc_class = "article", packages = [""], 
            title = "", author = "", 
            custom = []
    ):
        # // Base Document
        self.data = (
            f"\\documentclass{{{doc_class}}}" + 
            "".join(f"\\usepackage{{{p}}}" for p in packages)
        )

        # // Top Title and Author if maketitle
        if len(title) > 0:
            self.add(f"\\title{{{title}}}\\author{{{author}}}")
        
        # // Custom args and begin document
        if len(custom) > 0:
            self.add(" ".join(l for l in custom))
        self.add("\\begin{document}")
    
        # // If title, maketitle
        if len(title) > 0:
            self.add("\\maketitle")
        open(f"{self.file_dir}/build/{self.build_file}", "w").write("")
    
    # // Update the contents inside the provided .tex file
    def done(self):
        open(f"{self.file_dir}/build/{self.build_file}", "w").write(self.data + "\\end{document}")
    
    # // Update the data string
    def add(self, data: str):
        self.data += str(data)
        
    # // Create a new table element in the pdf
    def table(self, columns: int, headers: list[str], data: list[any]):
        self.add(Table(columns=columns, headers=headers, data=data))
    
    # // Create a new text element in the pdf
    def text(self, content: str = "", bold: bool = False, italic: bool = False):
        self.add(Text(content=content, bold=bold, italic=italic))
    
    # // Create a new adjust (adjustwidth) element in the pdf
    def adjust(self, width: int = 0, margin: int = 0, items: list[any] = []):
        self.add(Adjust(width=width, margin=margin, items=items))
    
    # // Create a new header (/section) element in the pdf
    def header(self, content: str, enumerate: bool = False):
        enum: str = {True: '', False: '*'}[enumerate]
        self.add(Header(content=content, enumerate=enum))
    
    # // Create a new list element
    def list(self, type: str = "itemize", items: list[any] = []):
        self.add(List(type=type, items=items))
    
    # // Create a new flex element
    def flex(self, width: int = 0.33, items = []):
        self.add(Flex(width=width, items=items))
    
    # // Create a new vertical space element
    def vspace(self, size: int = 1):
        self.add(Space(type="v", size=size))
        
    # // Create a new horizontal space element
    def hspace(self, size: int = 1):
        self.add(Space(type="h", size=size))
    
    # // Add a new line
    def newline(self):
        self.add("\\newline")
    
    # // Add a new image
    def image(self, path: str, scale: int):
        path_split: list[str] = path.split("/")
        file_name: str = "".join(f"\\{a}" for a in path_split[1:len(path_split) - 1])
        if not os.path.exists(f"{self.file_dir}/build/{file_name}"):
            os.mkdir(f"{self.file_dir}/build/{file_name}")
        shutil.copy(path, f"{self.file_dir}/build/{file_name}")
        self.add(Image(path=path, scale=scale))

